"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const inquirer = __importStar(require("inquirer"));
const mssql_1 = __importDefault(require("mssql"));
const fs_1 = __importDefault(require("fs"));
const ora_1 = __importDefault(require("ora"));
const mssql_2 = require("../queries/mssql");
const config_1 = __importDefault(require("../common/config"));
const eums_1 = require("./eums");
class Init {
    options;
    constructor(options) {
        this.options = options;
    }
    /**
     * Spinner instance.
     */
    spinner = (0, ora_1.default)();
    /**
     * Invoke action.
     */
    invoke() {
        const prompt = inquirer.createPromptModule();
        return prompt(this.getQuestions(false))
            .then(async (answers) => {
            const sett = this.writeFiles(answers);
            const tempFolderSpinner = this.spinner.start(`Creating temp folder...`);
            await this.createFolderIfNotExists(`${sett.output.root}\\${sett.output.temps}`);
            tempFolderSpinner.stop();
            // connect to db
            const sqlSpinner = this.spinner.start(`Creating SQL database trigger...`);
            new mssql_1.default.ConnectionPool(sett.connection)
                .connect()
                .then((pool) => {
                pool.request().query((0, mssql_2.dropTriggerWrite)()).then(() => {
                    pool.request().query((0, mssql_2.triggerWrite)(answers.root, answers.server)).then(() => {
                        pool.request().query((0, mssql_2.enableTriggerWrite)());
                    }).then(() => {
                        sqlSpinner.stop();
                    });
                });
            })
                .catch((error) => {
                console.error(error);
            });
        });
    }
    /**
     * Get all applicable questions.
     *
     * @param conn Connection object to use for default values.
     */
    getQuestions(showWebConfig) {
        const questions = [
            {
                choices: () => this.getPathChoices(showWebConfig),
                message: 'Where would you like to store connections?',
                name: 'path',
                type: 'list',
            },
            {
                default: 'localhost',
                message: 'Server URL.',
                name: 'server',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: '1433',
                message: 'Server port.',
                name: 'port',
                type: 'number',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: 'database',
                message: 'Database name.',
                name: 'database',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: 'username',
                message: 'Login username.',
                name: 'user',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: 'super_secure_password',
                message: 'Login password.',
                name: 'password',
                type: 'password',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: '0.0.1',
                message: 'Current version.',
                name: 'currentVersion',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: 'dev',
                message: 'Connection name.',
                name: 'name',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: `${process.cwd()}`,
                message: 'Root folder for project.',
                name: 'root',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './procs',
                message: 'Procedure folder.',
                name: 'procs',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './data',
                message: 'Data folder.',
                name: 'data',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './functions',
                message: 'Functions folder.',
                name: 'functions',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './jobs',
                message: 'Jobs folder.',
                name: 'jobs',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './schema',
                message: 'schemas folder.',
                name: 'schemas',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './tables',
                message: 'Tables folder.',
                name: 'tables',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './triggers',
                message: 'Trigger folder.',
                name: 'triggers',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './types',
                message: 'Types folder.',
                name: 'types',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
            {
                default: './views',
                message: 'Views folder.',
                name: 'views',
                when: (answers) => answers.path !== eums_1.PathChoices.WebConfig,
            },
        ];
        return questions;
    }
    /**
     * Get all available configuration file path choices.
     *
     * @param showWebConfig Indicates if Web.config choice should be available.
     */
    getPathChoices(showWebConfig) {
        const choices = [
            {
                name: 'Main configuration file.',
                value: eums_1.PathChoices.SscConfig,
            },
            {
                name: 'Separate connections configuration file.',
                value: eums_1.PathChoices.ConnsConfig,
            },
        ];
        if (showWebConfig) {
            choices.push({
                name: 'Web.config file with connection strings.',
                value: eums_1.PathChoices.WebConfig,
            });
        }
        return choices;
    }
    /**
     * From configuration files(s) based on answers.
     *
     * @param answers Answers from questions.
     */
    writeFiles(answers) {
        const sett = {
            name: answers.name,
            connection: {
                server: answers.server,
                database: answers.database,
                port: answers.port,
                user: answers.user,
                password: answers.password,
            },
            output: {
                root: answers.root,
                procs: answers.procs,
                data: answers.data,
                functions: answers.functions,
                jobs: answers.jobs,
                schemas: answers.schemas,
                tables: answers.tables,
                triggers: answers.triggers,
                types: answers.types,
                views: answers.views,
                temps: 'temp_files',
            },
            currentVersion: answers.currentVersion,
            idempotency: {
                procs: "if-exists-drop",
                data: 'truncate',
                functions: 'if-exists-drop',
                jobs: 'if-exists-drop',
                tables: 'if-exists-drop',
                triggers: 'if-exists-drop',
                types: 'if-exists-drop',
                views: 'if-exists-drop',
                temps: 'if-exists-drop',
            },
            includeConstraintName: true,
            loadFromObject: null,
            loadFromString: null
        };
        if (answers.path === eums_1.PathChoices.ConnsConfig) {
            config_1.default.write({ settings: [sett] }, config_1.default.defaultConnectionsJsonFile);
        }
        else {
            config_1.default.write({ settings: [sett] });
        }
        return sett;
    }
    createFolderIfNotExists = (folderPath) => {
        if (!fs_1.default.existsSync(folderPath)) {
            fs_1.default.mkdirSync(folderPath);
            console.log(`Folder created: ${folderPath}`);
        }
        else {
            console.log(`Folder already exists: ${folderPath}`);
        }
    };
}
exports.default = Init;
//# sourceMappingURL=init.js.map