"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mssql_1 = __importDefault(require("mssql"));
const ora_1 = __importDefault(require("ora"));
const config_1 = __importDefault(require("../common/config"));
const file_utility_1 = __importDefault(require("../common/file-utility"));
const mssql_2 = __importDefault(require("../generators/mssql"));
const mssql_3 = require("../queries/mssql");
const chokidar_1 = __importDefault(require("chokidar"));
const fs_1 = __importDefault(require("fs"));
const path = __importStar(require("path"));
class PullSingle {
    name;
    options;
    constructor(name, options) {
        this.name = name;
        this.options = options;
    }
    /**
     * Spinner instance.
     */
    spinner = (0, ora_1.default)();
    /**
     * Invoke action.
     */
    invoke() {
        const config = new config_1.default(this.options.config);
        const sett = config.getSetting(this.name);
        let temps = '';
        if (sett.output.temps != false) {
            const dir = path.join(sett.output.root, sett.output.temps);
            if (!fs_1.default.existsSync(dir) && sett.output.temps !== '') {
                fs_1.default.mkdirSync(dir);
            }
            temps = path.join(dir);
        }
        console.log('Listening to directory ' + temps);
        chokidar_1.default
            .watch(temps, { ignored: /^\./, persistent: true })
            .on('add', function (path) {
            const pathArray = path.split('\\');
            const fileArray = pathArray[pathArray.length - 1].split('.');
            const storedProcedureName = fileArray[0];
            const type = fileArray[1];
            console.log('Stored Procedure', storedProcedureName, 'has been triggered');
            // connect to db
            return new mssql_1.default.ConnectionPool(sett.connection)
                .connect()
                .then((pool) => {
                const queries = [
                    pool.request().query((0, mssql_3.objectRead)(type, storedProcedureName)),
                    pool.request().query(mssql_3.permissionsRead),
                ];
                return Promise.all(queries)
                    .then((results) => {
                    results[1].recordset;
                    return results;
                })
                    .then((results) => {
                    pool.close();
                    fs_1.default.unlink(path, (err) => {
                        if (err) {
                            console.error(err);
                        }
                    });
                    return results;
                });
            })
                .then((results) => writeFiles(config, sett, results, storedProcedureName, type))
                .catch((error) => {
                console.error(error);
            });
        })
            .on('change', function (path) {
            console.log('File', path, 'has been changed');
        })
            .on('unlink', function (path) {
            console.log('File', path, 'has been removed');
        })
            .on('error', function (error) {
            console.error('Error happened', error);
        });
    }
}
exports.default = PullSingle;
function writeFiles(config, sett, results, name, type) {
    // note: array order MUST match query promise array
    const objects = results[0].recordset;
    const permissions = results[1].recordset
        ? results[1].recordset
        : [];
    const generator = new mssql_2.default(config);
    const file = new file_utility_1.default(config, sett);
    let content;
    switch (type) {
        // stored procedures
        case 'P':
            name = `${name}.sql`;
            content = generator.storedProcedure(objects[0], sett);
            content += generator.permissions(permissions, name.split('.')[0]);
            file.write(`${sett.currentVersion}/${sett.output.procs}`, name, content);
            break;
        // views
        case 'V':
            name = `${name}.sql`;
            content = generator.view(objects[0], sett);
            file.write(sett.output.views, name, content);
            break;
        // functions
        case 'TF':
        case 'IF':
        case 'FN':
            name = `${name}.sql`;
            content = generator.function(objects[0], sett);
            file.write(sett.output.functions, name, content);
            break;
        default:
            break;
    }
    file.writeUpdate(generator);
    const msg = file.finalize();
    console.log(msg);
}
//# sourceMappingURL=pull-single.js.map