"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const sql = __importStar(require("mssql"));
const ora_1 = __importDefault(require("ora"));
const config_1 = __importDefault(require("../common/config"));
const file_utility_1 = __importDefault(require("../common/file-utility"));
const mssql_1 = __importDefault(require("../generators/mssql"));
const mssql_2 = require("../queries/mssql");
class Pull {
    name;
    options;
    constructor(name, options) {
        this.name = name;
        this.options = options;
    }
    /**
     * Spinner instance.
     */
    spinner = (0, ora_1.default)();
    /**
     * Invoke action.
     */
    invoke() {
        const config = new config_1.default(this.options.config);
        const sett = config.getSetting(this.name);
        this.spinner.start(`Pulling from ${chalk_1.default.blue(sett.connection.server)} ...`);
        // connect to db
        return new sql.ConnectionPool(sett.connection)
            .connect()
            .then((pool) => {
            const queries = [
                pool.request().query(mssql_2.objectsRead),
                pool.request().query(mssql_2.tablesRead),
                pool.request().query(mssql_2.columnsRead),
                pool.request().query(mssql_2.primaryKeysRead),
                pool.request().query(mssql_2.foreignKeysRead),
                pool.request().query(mssql_2.indexesRead),
                pool.request().query(mssql_2.typesRead),
            ];
            if (sett.output.jobs) {
                queries.push(pool.request().query((0, mssql_2.jobsRead)(sett.connection.database)), pool.request().query((0, mssql_2.jobStepsRead)(sett.connection.database)), pool.request().query((0, mssql_2.jobSchedulesRead)()));
            }
            else {
                queries.push(null, null, null);
            }
            queries.push(pool.request().query(mssql_2.permissionsRead));
            return Promise.all(queries)
                .then((results) => {
                results[1].recordset;
                return results;
            })
                .then((results) => {
                pool.close();
                return results;
            });
        })
            .then((results) => this.writeFiles(config, sett, results))
            .catch((error) => {
            console.error(error);
            this.spinner.fail(error);
        });
    }
    /**
     * Write all files to the file system based on `results`.
     *
     * @param config Current configuration to use.
     * @param results Array of data sets from SQL queries.
     */
    writeFiles(config, sett, results) {
        // note: array order MUST match query promise array
        const objects = results[0].recordset;
        const tables = results[1].recordset;
        const columns = results[2].recordset;
        const primaryKeys = results[3].recordset;
        const foreignKeys = results[4].recordset;
        const indexes = results[5].recordset;
        const types = results[6].recordset;
        const jobs = results[7] ? results[7].recordset : [];
        const jobSteps = results[8] ? results[8].recordset : [];
        const jobSchedules = results[9]
            ? results[9].recordset
            : [];
        const permissions = results[10].recordset
            ? results[10].recordset
            : [];
        const data = results.slice(11);
        const generator = new mssql_1.default(config);
        const file = new file_utility_1.default(config, sett);
        // schemas
        tables
            .map((item) => item.schema)
            .filter((value, index, array) => array.indexOf(value) === index)
            .map((value) => ({ name: value }))
            .forEach((item) => {
            const name = `${item.name}.sql`;
            const content = generator.schema(item);
            file.write(sett.output.schemas, name, content);
        });
        // stored procedures
        objects
            .filter((item) => item.type.trim() === 'P')
            .forEach((item) => {
            const name = `${item.name}.sql`;
            let content = generator.storedProcedure(item, sett);
            content += generator.permissions(permissions, name);
            file.write(sett.output.procs, name, content);
        });
        // views
        objects
            .filter((item) => item.type.trim() === 'V')
            .forEach((item) => {
            const name = `${item.schema}.${item.name}.sql`;
            const content = generator.view(item, sett);
            file.write(sett.output.views, name, content);
        });
        // functions
        objects
            .filter((item) => ['TF', 'IF', 'FN'].indexOf(item.type.trim()) !== -1)
            .forEach((item) => {
            const name = `${item.schema}.${item.name}.sql`;
            const content = generator.function(item, sett);
            file.write(sett.output.functions, name, content);
        });
        // triggers
        objects
            .filter((item) => item.type.trim() === 'TR')
            .forEach((item) => {
            const name = `${item.schema}.${item.name}.sql`;
            const content = generator.trigger(item, sett);
            file.write(sett.output.triggers, name, content);
        });
        // tables
        tables.forEach((item) => {
            const name = `${item.schema}.${item.name}.sql`;
            const content = generator.table(item, columns, primaryKeys, foreignKeys, indexes, sett);
            file.write(sett.output.tables, name, content);
        });
        // types
        types
            .filter((item) => !item.type)
            .forEach((item) => {
            const name = `${item.schema}.${item.name}.sql`;
            const content = generator.type(item, sett);
            file.write(sett.output.types, name, content);
        });
        // table types
        types
            .filter((item) => item.type && item.type.trim() === 'TT')
            .forEach((item) => {
            const name = `${item.schema}.${item.name}.sql`;
            const content = generator.tableType(item, columns, sett);
            file.write(sett.output.types, name, content);
        });
        // data
        data.forEach((item) => {
            const name = `${item.schema}.${item.name}.sql`;
            const content = generator.data(item, sett);
            file.write(sett.output.data, name, content);
        });
        // jobs
        jobs.forEach((item) => {
            const steps = jobSteps.filter((x) => x.job_id === item.job_id);
            const schedules = jobSchedules.filter((x) => x.job_id === item.job_id);
            const name = `${item.name}.sql`;
            const content = generator.job(item, steps, schedules, sett);
            file.write(sett.output.jobs, name, content);
        });
        const msg = file.finalize();
        this.spinner.succeed(msg);
    }
}
exports.default = Pull;
//# sourceMappingURL=pull.js.map