"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const fs = __importStar(require("fs-extra"));
const glob = __importStar(require("glob"));
const inquirer = __importStar(require("inquirer"));
const sql = __importStar(require("mssql"));
const ora_1 = __importDefault(require("ora"));
const os_1 = require("os");
class Push {
    name;
    options;
    constructor(name, options) {
        this.name = name;
        this.options = options;
    }
    /**
     * Spinner instance.
     */
    spinner = (0, ora_1.default)();
    /**
     * Invoke actions.
     */
    invoke() {
        const prompt = inquirer.createPromptModule();
        return prompt([
            {
                message: [
                    `${chalk_1.default.yellow('WARNING!')} All local SQL files will be executed against the requested database.`,
                    'This can not be undone!',
                    'Make sure to backup your database first.',
                    os_1.EOL,
                    'Are you sure you want to continue?',
                ].join(' '),
                name: 'continue',
                type: 'confirm',
                when: !this.options.skip,
            },
        ])
            .then((answers) => {
            if (answers.continue === false) {
                throw new Error('Command aborted!');
            }
        })
            .then(() => {
            this.spinner.succeed('Successfully pushed!');
        })
            .catch((error) => {
            this.spinner.fail(error.message);
        });
    }
    /**
     * Execute all files against database.
     *
     * @param config Configuration used to execute commands.
     * @param conn Connection used to execute commands.
     */
    batch(config, sett) {
        const files = this.getFilesOrdered(sett);
        let promise = new sql.ConnectionPool(sett.connection).connect();
        this.spinner.start(`Pushing to ${chalk_1.default.blue(sett.connection.server)} ...`);
        files.forEach((file) => {
            const content = fs.readFileSync(file, 'utf8');
            const statements = content.split('GO' + os_1.EOL);
            statements.forEach((statement) => {
                promise = promise.then((pool) => {
                    return pool
                        .request()
                        .batch(statement)
                        .then(() => pool);
                });
            });
        });
        return promise;
    }
    /**
     * Get all SQL files in correct execution order.
     *
     * @param config Configuration used to search for connection.
     */
    getFilesOrdered(setting) {
        const output = [];
        const directories = [
            setting.output.schemas,
            setting.output.tables,
            setting.output.types,
            setting.output.views,
            setting.output.functions,
            setting.output.procs,
            setting.output.triggers,
            setting.output.data,
            setting.output.jobs
        ];
        directories.forEach((dir) => {
            if (dir) {
                const files = glob.sync(`${setting.output.root}/${dir}/**/*.sql`);
                output.push(...files);
            }
        });
        return output;
    }
}
exports.default = Push;
//# sourceMappingURL=push.js.map