"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mssql_1 = __importDefault(require("mssql"));
const path_1 = __importDefault(require("path"));
const config_1 = __importDefault(require("../common/config"));
const fs_1 = __importDefault(require("fs"));
const chokidar_1 = __importDefault(require("chokidar"));
const mssql_2 = require("../queries/mssql");
const mssql_3 = __importDefault(require("../generators/mssql"));
const file_utility_1 = __importDefault(require("../common/file-utility"));
class Start {
    options;
    constructor(options) {
        this.options = options;
    }
    /**
     * Invoke action.
     */
    invoke() {
        const config = new config_1.default(this.options.config);
        const setts = config.getSettings();
        setts.forEach((sett) => {
            let temps = '';
            if (!fs_1.default.existsSync(sett.output.root)) {
                fs_1.default.mkdirSync(sett.output.root);
            }
            if (sett.output.temps != false) {
                const dir = path_1.default.join(sett.output.root, sett.output.temps);
                if (!fs_1.default.existsSync(dir) && sett.output.temps !== '') {
                    fs_1.default.mkdirSync(dir);
                }
                temps = path_1.default.join(dir);
            }
            console.log('Listening to directory ' + temps);
            chokidar_1.default
                .watch(temps, { ignored: /^\./, persistent: true })
                .on('add', function (path) {
                const pathArray = path.split('\\');
                const fileArray = pathArray[pathArray.length - 1].split('.');
                const storedProcedureName = fileArray[0];
                const type = fileArray[1];
                console.log('Stored Procedure', storedProcedureName, 'has been triggered');
                // connect to db
                return new mssql_1.default.ConnectionPool(sett.connection)
                    .connect()
                    .then((pool) => {
                    const queries = [
                        pool.request().query((0, mssql_2.objectRead)(type, storedProcedureName)),
                        pool.request().query(mssql_2.permissionsRead),
                    ];
                    return Promise.all(queries)
                        .then((results) => {
                        results[1].recordset;
                        return results;
                    })
                        .then((results) => {
                        pool.close();
                        fs_1.default.unlink(path, (err) => {
                            if (err) {
                                console.error(err);
                            }
                        });
                        return results;
                    });
                })
                    .then((results) => writeFiles(config, sett, results, storedProcedureName, type))
                    .catch((error) => {
                    console.error(error);
                });
            })
                .on('change', function (path) {
                console.log('File', path, 'has been changed');
            })
                .on('error', function (error) {
                console.error('Error happened', error);
            });
        });
    }
}
exports.default = Start;
async function writeFiles(config, sett, results, name, type) {
    // note: array order MUST match query promise array
    const objects = results[0].recordset;
    const permissions = results[1].recordset
        ? results[1].recordset
        : [];
    const generator = new mssql_3.default(config);
    const file = new file_utility_1.default(config, sett);
    let content;
    switch (type) {
        // stored procedures
        case 'P':
            name = `${name}.sql`;
            content = generator.storedProcedure(objects[0], sett);
            content += generator.permissions(permissions, name.split('.')[0]);
            file.write(`${sett.currentVersion}/${sett.output.procs}`, name, content);
            file.write(`${sett.output.procs}`, name, content);
            break;
        // views
        case 'V':
            name = `${name}.sql`;
            content = generator.view(objects[0], sett);
            file.write(`${sett.currentVersion}/${sett.output.views}`, name, content);
            file.write(sett.output.views, name, content);
            break;
        // functions
        case 'TF':
        case 'IF':
        case 'FN':
            name = `${name}.sql`;
            content = generator.function(objects[0], sett);
            file.write(`${sett.currentVersion}/${sett.output.functions}`, name, content);
            file.write(sett.output.functions, name, content);
            break;
        default:
            break;
    }
    await file.writeUpdate(generator);
}
//# sourceMappingURL=start.js.map