"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
/**
 * File checksum cache.
 */
class Cache {
    /**
     * Default cache file.
     */
    static defaultCacheFile = 'cache.json';
    constructor(config, sett) {
        this.config = config;
        this.sett = sett;
    }
    /**
     * File checksums.
     */
    files = {};
    /**
     * Current configuration.
     */
    config;
    /**
     * Current setting.
     */
    sett;
    /**
     * Load configuration options from file.
     */
    load() {
        if (!this.doesDefaultExist()) {
            return;
        }
        try {
            const file = path.join(/*this.config.getRoot()*/ this.sett.output.root, Cache.defaultCacheFile);
            const cache = fs.readJsonSync(file);
            this.files = cache.files;
        }
        catch (error) {
            console.error(`Could not parse cache file. Try deleting the existing ${Cache.defaultCacheFile} file!`);
            process.exit();
        }
    }
    /**
     * Check if a `newSum` is different from the existing file's checksum.
     *
     * @param file File to check.
     * @param newSum New checksum value.
     */
    didChange(file, newSum) {
        if (!this.files) {
            return true;
        }
        const oldSum = this.files[file];
        if (!oldSum) {
            return true;
        }
        return newSum !== oldSum;
    }
    /**
     * Add file checksum to cache.
     *
     * @param file File to check.
     * @param newSum New checksum value.
     */
    add(file, newSum) {
        if (!file || !newSum) {
            return;
        }
        this.files[file] = newSum;
    }
    /**
     * Write a config file with provided configuration.
     */
    write() {
        const file = path.join(/*this.config.getRoot()*/ this.sett.output.root, Cache.defaultCacheFile);
        const content = { files: this.files };
        fs.writeJson(file, content, { spaces: 2 });
    }
    /**
     * Check if default cache file exists.
     */
    doesDefaultExist() {
        const file = path.join(/*this.config.getRoot()*/ this.sett.output.root, Cache.defaultCacheFile);
        return fs.existsSync(file);
    }
}
exports.default = Cache;
//# sourceMappingURL=cache.js.map