"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const ts_util_is_1 = require("ts-util-is");
/**
 * Configuration options.
 */
class Config {
    /**
     * Default connections JSON file.
     */
    static defaultConnectionsJsonFile = 'ssc-connections.json';
    /**
     * Default Web.config file.
     */
    static defaultWebConfigFile = 'Web.config';
    /**
     * Default configuration file.
     */
    static defaultConfigFile = 'ssc.json';
    /**
     * Write a config file with provided configuration.
     *
     * @param config Configuration object to write.
     * @param file Configuration file to write to.
     */
    static write(config, file) {
        const configFile = path.join(process.cwd(), file || Config.defaultConfigFile);
        const content = JSON.stringify(config, null, 2);
        fs.outputFile(configFile, content, (error) => {
            if (error) {
                return console.error(error);
            }
            console.log('Config file created!');
        });
    }
    /**
     * Update a config file with provided configuration.
     *
     * @param settings Settings to update config with.
     * @param file Configuration file to write to.
     */
    static update(settings, file) {
        const config = new Config();
        config.settings = settings;
        const configFile = path.join(process.cwd(), file || Config.defaultConfigFile);
        const content = JSON.stringify(config, null, 2);
        fs.outputFile(configFile, content, (error) => {
            if (error) {
                return console.error(error);
            }
            console.log('Config file updated!');
        });
    }
    /**
     * Check if default configuration file exists.
     */
    static doesDefaultExist() {
        return fs.existsSync(Config.defaultConfigFile);
    }
    /**
     * Safely get connections from a Web.config file.
     *
     * @param file Relative path to Web.config file.
     */
    static getConnectionsFromWebConfig() {
        const conns = '';
        //   return conns.length ? conns : undefined;
        return conns;
    }
    constructor(file) {
        this.load(file);
    }
    settings = [];
    /**
     * Get a setting by name, or the first available if `name` is not provided.
     *
     * @param name Optional connection `name` to get.
     */
    getSetting(name) {
        const setts = this.getSettings();
        let sett;
        let error;
        if (name) {
            sett = setts.find((item) => item.name.toLocaleLowerCase() === name.toLowerCase());
            error = `Could not find settings by name '${name}'!`;
        }
        else {
            sett = setts[0];
            error = 'Could not find default setting!';
        }
        if (!sett) {
            console.error(error);
            process.exit();
        }
        sett.connection = Object.assign(sett.connection, {
            options: {
                // https://github.com/tediousjs/tedious/releases/tag/v7.0.0
                enableArithAbort: true,
                cryptoCredentialsDetails: {
                    minVersion: 'TLSv1',
                },
                encrypt: false,
            },
            requestTimeout: 5000,
        });
        return sett;
    }
    /**
     * Safely get all settings.
     */
    getSettings() {
        if (!(0, ts_util_is_1.isString)(this.settings)) {
            return this.settings;
        }
        return this.getSettingsFromJson(this.settings);
    }
    /**
     * Load configuration options from file.
     *
     * @param file Configuration file to load.
     */
    load(file) {
        const configFile = path.join(file || Config.defaultConfigFile);
        try {
            const config = fs.readJsonSync(configFile);
            this.settings = config.settings || this.settings;
        }
        catch (error) {
            console.error('Could not find or parse config file. You can use the `init` command to create one!');
            process.exit();
        }
    }
    /**
     * Safely get settings from a JSON file.
     *
     * @param file Relative path to settings JSON file.
     */
    getSettingsFromJson(file) {
        const jsonFile = path.join(process.cwd(), file);
        try {
            const config = fs.readJsonSync(jsonFile);
            return config.settings;
        }
        catch (error) {
            console.error('Could not find or parse settings config file!');
            process.exit();
        }
    }
}
exports.default = Config;
//# sourceMappingURL=config.js.map