"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mock_fs_1 = __importDefault(require("mock-fs"));
const config_1 = __importDefault(require("./config"));
describe('Config class', () => {
    const name = 'dev';
    const server = 'localhost';
    const port = 1433;
    const database = 'awesome-db';
    const user = 'example';
    const password = 'qwerty';
    const connection = {
        database,
        password,
        port,
        server,
        user
    };
    const files = ['dbo.*'];
    const data = ['dbo.LookupTable'];
    const output = { root: './my-database' };
    const idempotency = { triggers: false };
    beforeEach(() => {
        mock_fs_1.default.restore();
    });
    afterAll(() => {
        mock_fs_1.default.restore();
    });
    describe('doesDefaultExist method', () => {
        it('should return true if file exists', () => {
            const file = config_1.default.defaultConfigFile;
            (0, mock_fs_1.default)({
                [file]: '',
            });
            const value = config_1.default.doesDefaultExist();
            expect(value).toEqual(true);
        });
    });
    describe('doesDefaultExist method', () => {
        it('should return false if file not exists', () => {
            (0, mock_fs_1.default)({});
            let value;
            // https://github.com/tschaub/mock-fs/issues/256
            try {
                value = config_1.default.doesDefaultExist();
            }
            catch (ex) {
                value = false;
            }
            expect(value).toEqual(false);
        });
    });
    describe('getConnectionsFromWebConfig method', () => {
        it('should return connections if default web.config exists', () => {
            const file = config_1.default.defaultWebConfigFile;
            (0, mock_fs_1.default)({
                [file]: `
          <?xml version="1.0" encoding="utf-8"?>
          <configuration>
            <connectionStrings>
            <add
              name="${name}"
              connectionString="server=${server};database=${database};uid=${user};password=${password};" />
            </connectionStrings>
          </configuration>
        `,
            });
        });
    });
    describe('constructor', () => {
        it('should load from default file', () => {
            const file = config_1.default.defaultConfigFile;
            (0, mock_fs_1.default)({
                [file]: JSON.stringify({
                    connections: [connection],
                    data,
                    files,
                    idempotency,
                    output,
                }),
            });
        });
        it('should load from specified file', () => {
            const file = 'override-example.json';
            (0, mock_fs_1.default)({
                [file]: JSON.stringify({
                    connections: [connection],
                    data,
                    files,
                    idempotency,
                    output,
                }),
            });
        });
    });
    describe('getConnection method', () => {
        it('should return first connection', () => {
            const file = config_1.default.defaultConfigFile;
            (0, mock_fs_1.default)({
                [file]: JSON.stringify({
                    connections: [connection],
                }),
            });
        });
        it('should return connection by name', () => {
            const file = config_1.default.defaultConfigFile;
            (0, mock_fs_1.default)({
                [file]: JSON.stringify({
                    connections: [connection],
                }),
            });
        });
    });
    describe('getConnections method', () => {
        it('should return all conneections', () => {
            const file = config_1.default.defaultConfigFile;
            (0, mock_fs_1.default)({
                [file]: JSON.stringify({
                    connections: [connection],
                }),
            });
        });
    });
});
//# sourceMappingURL=config.spec.js.map