"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var connection_1 = __importDefault(require("./connection"));
describe('Connection class', function () {
    var name = 'dev';
    var server = 'localhost';
    var port = 1433;
    var database = 'awesome-db';
    var user = 'example';
    var password = 'qwerty';
    describe('loadFromString method', function () {
        it('should hydrate from string (format #1)', function () {
            var str = "server=".concat(server, ";database=").concat(database, ";uid=").concat(user, ";password=").concat(password, ";");
            var conn = new connection_1.default();
            conn.loadFromString(name, str);
            expect(conn.name).toEqual(name);
            expect(conn.server).toEqual(server);
            expect(conn.port).toBeUndefined();
            expect(conn.database).toEqual(database);
            expect(conn.user).toEqual(user);
            expect(conn.password).toEqual(password);
        });
        it('should hydrate from string (format #2)', function () {
            var str = "server=".concat(server, ";database=").concat(database, ";uid=").concat(user, ";pwd=").concat(password, ";");
            var conn = new connection_1.default();
            conn.loadFromString(name, str);
            expect(conn.name).toEqual(name);
            expect(conn.server).toEqual(server);
            expect(conn.port).toEqual(undefined);
            expect(conn.database).toEqual(database);
            expect(conn.user).toEqual(user);
            expect(conn.password).toEqual(password);
        });
        it('should hydrate from string with port', function () {
            var str = "server=".concat(server, ",").concat(port, ";database=").concat(database, ";uid=").concat(user, ";password=").concat(password, ";");
            var conn = new connection_1.default();
            conn.loadFromString(name, str);
            expect(conn.name).toEqual(name);
            expect(conn.server).toEqual(server);
            expect(conn.port).toEqual(port);
            expect(conn.database).toEqual(database);
            expect(conn.user).toEqual(user);
            expect(conn.password).toEqual(password);
        });
    });
    describe('loadFromObject method', function () {
        it('should hydrate from object', function () {
            var conn = new connection_1.default();
            conn.loadFromObject({
                database: database,
                password: password,
                port: port,
                server: server,
                user: user,
            });
            expect(conn.name).toEqual(name);
            expect(conn.server).toEqual(server);
            expect(conn.port).toEqual(port);
            expect(conn.database).toEqual(database);
            expect(conn.user).toEqual(user);
            expect(conn.password).toEqual(password);
        });
    });
});
//# sourceMappingURL=connection.spec.js.map