"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const checksum_1 = __importDefault(require("checksum"));
const eol = __importStar(require("eol"));
const filenamify_1 = __importDefault(require("filenamify"));
const fs = __importStar(require("fs-extra"));
const glob = __importStar(require("glob"));
const path = __importStar(require("path"));
const cache_1 = __importDefault(require("./cache"));
const async = require('async');
/**
 * File system interaction and tracking.
 */
class FileUtility {
    constructor(config, sett) {
        this.config = config;
        this.sett = sett;
        this.existingCache = new cache_1.default(config, sett);
        this.newCache = new cache_1.default(config, sett);
        this.load();
    }
    /**
     * Current configuration.
     */
    config;
    /**
     * Current settings.
     */
    sett;
    /**
     * Existing files.
     */
    existingFiles;
    /**
     * Existing cache.
     */
    existingCache;
    /**
     * Cache generated during file writing.
     */
    newCache;
    /**
     * Counts about files written / removed.
     */
    stats = {
        added: 0,
        removed: 0,
        updated: 0,
    };
    /**
     * Write file to the file system.
     *
     * @param file Directory to write, relative to root.
     * @param file File name to write to.
     * @param content File content to write.
     */
    write(dir, file, content) {
        if (dir === false) {
            return;
        }
        // remove unsafe characters
        file = (0, filenamify_1.default)(file);
        file = path.join(this.sett.output.root, dir, file);
        content = eol.auto(content);
        content = content.trim();
        const cacheKey = this.normalize(file);
        const cacheValue = (0, checksum_1.default)(content);
        this.newCache.add(cacheKey, cacheValue);
        if (!this.doesExist(file)) {
            this.stats.added++;
        }
        else if (this.existingCache.didChange(cacheKey, cacheValue)) {
            this.stats.updated++;
        }
        fs.outputFileSync(file, content);
        //this.markAsWritten(file);
    }
    /**
     * Delete all paths remaining in `existing`.
     */
    finalize() {
        this.existingFiles.forEach((file) => {
            this.stats.removed++;
            fs.removeSync(file);
        });
        this.newCache.write();
        const added = chalk_1.default.green(this.stats.added.toString());
        const updated = chalk_1.default.cyan(this.stats.updated.toString());
        const removed = chalk_1.default.red(this.stats.removed.toString());
        return `Successfully added ${added}, updated ${updated}, and removed ${removed} files.`;
    }
    /**
     * Check if a file existed.
     *
     * @param file File path to check.
     */
    doesExist(file) {
        if (!this.existingFiles || !this.existingFiles.length) {
            return false;
        }
        file = this.normalize(file);
        const index = this.existingFiles.indexOf(file);
        return index !== -1;
    }
    /**
     * Remove `file` from `existing`, if it exists.
     *
     * @param file File path to check.
     */
    markAsWritten(file, currentVerson) {
        if (!file) {
            return;
        }
        file = this.normalize(file);
        console.log(this.existingFiles);
        console.log(file);
        this.existingFiles.splice(0, 1);
        this.existingFiles.push(file);
        console.log(this.existingFiles);
    }
    /**
     * Normalize file path for comparison.
     *
     * @param file File path to normalize.
     */
    normalize(file) {
        const root = this.sett.output.root;
        if (root.startsWith('./') && !file.startsWith('./')) {
            file = `./${file}`;
        }
        return file.replace(/\\/g, '/');
    }
    /**
     * Load existing files and cache for comparison.
     */
    load() {
        this.existingFiles = glob.sync(`${this.sett.output.root}/**/*.sql`);
        this.existingCache.load();
    }
    /**
     * Write update file to the file system.
     *
     */
    writeUpdate(generator) {
        const { storedProcedureDirectory, functionDirectory, filesDestinationDirectory } = this.getDirectories();
        let files = [];
        if (!fs.existsSync(storedProcedureDirectory)) {
            fs.mkdirSync(storedProcedureDirectory);
        }
        if (!fs.existsSync(functionDirectory)) {
            fs.mkdirSync(functionDirectory);
        }
        new Promise((resolve, _reject) => {
            if (this.sett.output.procs != false) {
                fs.readdir(storedProcedureDirectory, (err, filenames) => {
                    files = this.mergeArrays(files, filenames.map((file) => path.join(storedProcedureDirectory, file)));
                    resolve(files);
                });
                return files;
            }
        })
            .then(() => {
            if (this.sett.output.functions != false) {
                fs.readdir(functionDirectory, (err, filenames) => {
                    if (err) {
                        console.error(err);
                    }
                    files = this.mergeArrays(files, filenames.map((file) => path.join(functionDirectory, file)));
                    return files;
                });
            }
        })
            .finally(() => {
            async.map(files, fs.readFile, (err, results) => {
                if (err) {
                    console.error(err);
                    return err;
                }
                const content = generator.upgradeAudit(this.sett.name, this.sett.currentVersion, 'Object');
                results.splice(0, 0, content);
                //Write the joined results to destination
                fs.writeFile(filesDestinationDirectory, results.join('\n\n\n\n'), (err) => {
                    if (err) {
                        console.error(err);
                        return err;
                    }
                    console.log(chalk_1.default.bgMagenta.white.bold('\nUpdate file has been generated.'));
                });
            });
        })
            .catch((err) => {
            console.error(err);
        });
    }
    mergeArrays = function (arr1, arr2) {
        if (arr2.length == 1) {
            arr1.push(arr2[0]);
        }
        else if (arr2.length > 1) {
            arr2.forEach(function (item) {
                arr1.push(item);
            });
        }
        return arr1;
    };
    getDirectories() {
        const storedProcedureDirectory = path.join(this.sett.output.root, this.sett.currentVersion, this.sett.output.procs.toString());
        const functionDirectory = path.join(this.sett.output.root, this.sett.currentVersion, this.sett.output.functions.toString());
        const filesDestinationDirectory = path.join(this.sett.output.root, this.sett.currentVersion, `${this.sett.name} Upgrade ${this.sett.currentVersion} - 3 Objects.sql`);
        return { storedProcedureDirectory, functionDirectory, filesDestinationDirectory };
    }
}
exports.default = FileUtility;
//# sourceMappingURL=file-utility.js.map