"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const { program } = require('commander');
const update_notifier_1 = __importDefault(require("update-notifier"));
const pkg = require("../package.json");
const init_1 = __importDefault(require("./commands/init"));
const list_1 = __importDefault(require("./commands/list"));
const pull_1 = __importDefault(require("./commands/pull"));
const pull_single_1 = __importDefault(require("./commands/pull-single"));
const push_1 = __importDefault(require("./commands/push"));
const start_1 = __importDefault(require("./commands/start"));
const bump_1 = __importDefault(require("./commands/bump"));
async function main() {
    program
        .command('init')
        .description('Create default config file.')
        .option('-f, --force', 'Overwrite existing config file, if present.')
        .option('-s, --skip', 'Use defaults only and skip the option prompts.')
        .option('-w, --webconfig [value]', 'Relative path to Web.config file.')
        .action((options) => {
        const action = new init_1.default(options);
        return action.invoke();
    });
    program
        .command('list')
        .alias('ls')
        .description('List all available connections.')
        .option('-c, --config [value]', 'Relative path to config file.')
        .action((options) => {
        const action = new list_1.default(options);
        return action.invoke();
    });
    program
        .command('start')
        .description('Starts the service that tracks changes in folders used by ssc.')
        .option('-c, --config [value]', 'Relative path to config file.')
        .action((options) => {
        const action = new start_1.default(options);
        return action.invoke();
    });
    program
        .command('bump')
        .alias('b')
        .description('Increase the version number that ssc outputs to.')
        .requiredOption('-nv, --newversion [value]', 'New version you wish to set for the connection')
        .requiredOption('-c, --conn [value]', 'The name of the connection you wish to update')
        .action((options) => {
        const action = new bump_1.default(options);
        return action.invoke();
    });
    program
        .command('pull [name]')
        .description('Generate SQL files for all tables, stored procedures, functions, etc.')
        .option('-c, --config [value]', 'Relative path to config file.')
        .action((name, options) => {
        const action = new pull_1.default(name, options);
        return action.invoke();
    });
    program
        .command('pull-single [name]')
        .description('Generate SQL files for a specific table, stored procedure, function, etc.')
        .option('-c, --config [value]', 'Relative path to config file.')
        .action((name, options) => {
        const action = new pull_single_1.default(name, options);
        return action.invoke();
    });
    program
        .command('push [name]')
        .description('Execute all scripts against the requested database.')
        .option('-c, --config [value]', 'Relative path to config file.')
        .option('-s, --skip', 'Skip user warning prompt.')
        .action((name, options) => {
        const action = new push_1.default(name, options);
        return action.invoke();
    });
    program.version(pkg.version);
    await program.parseAsync(process.argv);
}
(0, update_notifier_1.default)({ pkg }).notify();
main();
//# sourceMappingURL=index.js.map